#ifndef DIST_DISCRETE_H_
#define DIST_DISCRETE_H_

#include <distributions/Distribution.h>

/**
 * Abstract class for integer valued distributions whose support
 * is not a finite set.
 *
 * A subclass of DistDiscrete only has to implement the d,p,q, and
 * r virtual member functions. These are based on the d-p-q-r functions
 * provided by libRmath and, for most of the standard JAGS distributions,
 * are simply wrappers around them.
 *
 * @see DistFinite DistReal
 * @short Integer valued distributions *
 */
class DistDiscrete : public Distribution
{
public:
  DistDiscrete(std::string const &name, unsigned int npar);
  double logLikelihood(SArray const &x,
		       std::vector<SArray const *> const &parameters) const;
  void randomSample(SArray &x,
		    std::vector<SArray const *> const &parameters) const;
  /**
   * The lowest possible value that a random variable with these
   * parameters can take.
   */	
  long lowerLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * The highest possible value that a random variable with
   * these parameters can take.
   */
  long upperLimit(std::vector<SArray const *> const &parameters) const;
  /** 
   * Density function.
   * @param x value at which to evaluate the density
   * @param parameters Array of parameters
   * @param give_log Indicates whether to return log density. 
   */
  virtual double d(double x, std::vector<SArray const *> const &parameters, 
		   bool give_log) const = 0;
  /** 
   * Distribution function. 
   * @param x quantile at which to evaluate the distribution function
   * @param parameters Array of parameters
   * @param lower If true, return value is P[X <= x], otherwise
   * P[X > x]
   * @param give_log Indicates whether to return log probabability
   */
  virtual double p(double x, std::vector<SArray const *> const &parameters, bool lower,
		   bool give_log) const = 0;
  /** 
   * Quantile function. 
   * @param p probability at which to evaluate quantile
   * @param parameters Array of parameters
   * @param log_p Indicates whether p is given as log(p). 
   */
  virtual double q(double p, std::vector<SArray const *> const &parameters, bool lower,
		   bool log_p) const = 0;
  /**
   * Random number generation
   * @param parameters Array of parameters
   */
  virtual double r(std::vector<SArray const *> const &parameters) const = 0;
};

#endif /* DIST_DISCRETE_H_ */
