#ifndef DIST_REAL_H_
#define DIST_REAL_H_

#include <distributions/Distribution.h>

/**
 * Enumerates three possible ranges of support for a real valued
 * random variable
 *
 * DIST_REALS means support is on the whole real line
 *
 * DIST_POSITIVE_REAL means support on values > 0
 *
 * DIST_PROPORTIONS means support on values in (0,1)
 */
enum Support {DIST_REAL, DIST_POSITIVE_REAL, DIST_PROPORTION};

/**
 * Abstract class for real valued distributions
 *
 * A subclass of DistReal only has to implement the d,p,q, and
 * r virtual member functions. These are based on the d-p-q-r functions
 * provided by libRmath and, for most of the standard JAGS distributions
 * are simply wrappers around them.
 *
 * @short Real valued distributions
 */
class DistReal : public Distribution
{
  const Support _support;
  const int _lower;
  const int _upper;
public:
  /**
   * Constructor
   * @param name BUGS language name of distribution
   *
   * @param npar Number of parameters, excluding upper and lower bound
   *
   * @param support Support of distribution
   *
   * @param lower The support of some distributions depends on
   * the parameter values (e.g. Pareto, Uniform).  In order to
   * handle these distributions in the most general way, the
   * constructor has "lower" and "upper" parameters for the
   * indices of the parameters that define lower and upper
   * limits (starting from 1). Since this case is relatively
   * rare there is a default (0) meaning that the support is
   * independent of the parameters.  
   *
   * @param upper See lower
   */
  DistReal(std::string const &name, unsigned int npar,
	   Support support, int lower=0, int upper=0);
  double logLikelihood(SArray const &x,
		       std::vector<SArray const *> const &parameters) const;
  void randomSample(SArray &x,
		    std::vector<SArray const *> const &parameters) const;
  /**
   * Most real-valued distributions have only scalar parameters. For
   * convenience, a default checkParameterDim member function is provided
   * that checks returns true if the value and the parameters are all 
   * scalar.  Distributions that do not take scalar parameters need to
   * overload this
   */
  bool checkParameterDim (SArray const &value, std::vector<SArray const *> const &parameters) const;
  /**
   * The lowest possible value that a random variable with these
   * parameters can take.
   */
  double lowerLimit(std::vector<SArray const *> const &parameters) const;
  /**
   * The highest possible value that a random variable with
   * these parameters can take.
   */
  double upperLimit(std::vector<SArray const *> const &parameters) const;
  /** 
   * Density function.
   * @param x value at which to evaluate the density
   * @param parameters Array of parameters
   * @param give_log Indicates whether to return log density. 
   */
  virtual double d(double x, std::vector<SArray const *> const &parameters, 
		   bool give_log) const = 0;
  /** 
   * Distribution function. 
   * @param x quantile at which to evaluate the distribution function
   * @param parameters Array of parameters
   * @param lower If true, return value is P[X <= x]. Otherwise
   * P[X > x]
   * @param give_log Indicates whether to return log probabability
   */
  virtual double p(double x, std::vector<SArray const *> const &parameters, bool lower,
		   bool give_log) const = 0;
  /** 
   * Quantile function. 
   * @param p probability for which to evaluate quantile
   * @param parameters Array of parameters
   * @param log_p Indicates whether p is given as log(p). 
   */
  virtual double q(double p, std::vector<SArray const *> const &parameters, bool lower,
		   bool log_p) const = 0;
  /**
   * Random number generation
   * @param parameters Array of parameters
   */
  virtual double r(std::vector<SArray const *> const &parameters) const = 0;
};

DistReal const *asReal(Distribution const *dist);

#endif /* DIST_REAL_H_ */
