#ifndef DISTRIBUTION_H_
#define DISTRIBUTION_H_

#include <vector>
#include <string>
class SArray;

/**
 * An abstract base class for distributions.
 *
 * Distribution objects contain only constant data members and all
 * member functions are constant. Hence only one object needs to be
 * instantiated for each subclass.
 *
 * The DistTab class provides a convenient way of storing Distribution
 * objects and referencing them by name.
 * @see DistTab
 * @short Distribution 
 */
class Distribution
{
  const std::string _name;
  const unsigned int _npar;
public:
  /**
   * Constructor.
   * @param name name of the distribution as used in the BUGS language
   * @param npar number of parameters, excluding upper and lower bounds
   */
  Distribution(std::string const &name, unsigned int npar);
  virtual ~Distribution();
  /**
   * @returns the  BUGS language name of the distribution
   */
  std::string const &name() const;
  /**
   * @param x Array of values at which to evaluate the
   * likelihood.
   *
   * @param parameters Array of parameter values at which
   * to evaluate the likelihood. This vector should be of length
   * npar() + 2, where the last two elements are the lower and
   * upper bounds, or 0 if the distribution is unbounded.
   *
   * @returns the log likelihood.  If the likelihood should be
   * zero because x is inconsistent with the parameters, then
   * -DBL_MAX is returned.
   */
  virtual double logLikelihood(SArray const & x, std::vector<SArray const *> const &parameters) const = 0;
  /**
   * Draw a random sample from the distribution. 
   *
   * @param x Array to which the sample values are written
   *
   * @param parameters  Vector of parameter values at which
   * to evaluate the likelihood. This vector should be of length
   * npar() + 2, where the last two elements are the lower and
   * upper bounds, or 0 if the distribution is unbounded.
   *
   * @exception length_error 
   */
  virtual void randomSample(SArray &x, std::vector<SArray const *> const &parameters) const  = 0;
  /**
   * Checks that the length of the parameter vector is correct
   */
  bool checkParameterLength(std::vector<SArray const *> const &parameters) const;
  /**
   * Check that dimensions of the parameters are correct, and
   * consistent with the value.
   *
   * This function only needs to be run once for each parameter
   * vector. Thereafter, the values of the parameters will
   * change, but the dimensions will not.
   */
  virtual bool checkParameterDim (SArray const &value, std::vector<SArray const *> const &parameters) const = 0;
  /**
   * Check that the values of the parameters are consistent with
   * the distribution. For example, some distributions require
   * than certain parameters are positive, or lie in a given
   * range.
   *
   * This function assumes that checkParameterDim returns true.
   */
  virtual bool checkParameterValue (std::vector<SArray const *> const &parameters)
    const = 0;
  /**
   * @returns pointer to an SArray that contains the lower bound of
   * the distribution, or 0 if there is no lower bound.
   */
  SArray const *lowerBound(std::vector<SArray const *> const &parameters) 
    const;
  /**
   * @returns pointer to an SArray that contains the upper bound of
   * the distribution, or 0 if there is no upper bound.
   */
  SArray const *upperBound(std::vector<SArray const *> const &parameters) 
    const;
};

#endif /* DISTRIBUTION_H_ */
