#ifndef FUNCTION_H_
#define FUNCTION_H_

#include <string>
#include <vector>

class SArray;
class Index;

/**
 * An abstract base class for functions.
 *
 * Function objects contain only constant data members and all member
 * functions are constant. Hence only one object needs to be
 * instantiated for each subclass.
 *
 * @see FuncTab
 * @short Function
 */
class Function
{
  const std::string _name;
  const unsigned int _npar;
public:
  /**
   * Constructor.
   * @param name name of the function as used in the BUGS language
   * @param npar number of parameters. If npar == 0 then the function
   * takes a variable number of parameters.
   */
  Function (std::string const &name, unsigned int npar);
  virtual ~ Function ();
  /**
   * Returns the  BUGS language name of the function
   */
  std::string const &name () const;
  /**
   * Evaluates the function. This function assumes that checkDim and
   * checkParameterValue return true.
   *
   * @param value SArray which is modified to contain the result of
   * the evaluation 
   * @param args Vector of arguments
   */
  virtual void
  evaluate (SArray & value,
	    std::vector <SArray const *> const &args) const = 0;
  /**
   * Calculates what the dimension of the return value should be,
   * based on the arguments
   */
  virtual Index dim (std::vector <SArray const *> const &args) const = 0;
  /**
   * Checks whether dimensions of the arguments and the value are
   * conforming. This function assumes that checkParameterLength
   * returns true.
   */
  virtual bool
  checkDim (SArray const &value, std::vector <SArray const *> const &args)
    const = 0;
  /**
   * Checks whether the parameter values lie in the domain of
   * the function.  This assumes that the function checkDim
   * returns true.
   *
   * Since many functions do not need to check their parameter
   * values a default method is provided which always returns
   * true.
   */
  virtual bool
  checkParameterValue (std::vector <SArray const *> const &args) const;
  /**
   * Checks that parameter vector is of correct length
   */
  virtual bool 
  checkParameterLength (std::vector <SArray const *> const &args) const;
};

#endif /* FUNCTION_H_ */
