#ifndef SYMTAB_H_
#define SYMTAB_H_

#include <model/NodeArray.h>
#include <sarray/SArray.h>

#include <string>
#include <vector>
#include <map>

/**
 * @short Symbol table
 *
 * The SymTab class stores the names of variable being used in the
 * BUGS language representation of the model.  
 *
 * @see NodeArray
 */
class SymTab {
  std::map<std::string, NodeArray*> _varTable;
public:
  SymTab();
  /**
   * Adds an array variable to the symbol table. This creates a
   * NodeArray object of the given dimension and associates it
   * with the name, so it can be retrieved with a call to getVariable.
   * If no dimension is given, the variable is assumed to be scalar.
   */
  void addVariable(std::string const &name, Index const &dim = Index(1));
  /**
   * Returns a pointer to the  NodeArray associated with the given
   * name, or a NULL pointer if there is no such NodeArray.
   */
  NodeArray *getVariable(std::string const &name) const;
  /**
   * Adds all the nodes contained in all the NodeArray objects
   * in the SymTab to the given vector
   */
  void getNodes(std::vector<Node*> &nodes);
  /*
   * Write values from the data table to the nodes in SymTab with the
   * same name.  The observed flag indicates whether the data are
   * observed (hence the nodes are to be set as fixed) or not. In the
   * latter case, the values written are merely initial values that
   * may be overwritten.
   */
  void writeData(std::map<std::string, SArray> const &data_table,
		 bool observed);
  /*
   * Reads the current value of the nodes and writes the result
   * to the data table.
   *
   * @param data_table Data table.
   * @param observed Logical flag. If true, then the values of the
   * fixed nodes are written to the data table. If false, then the
   * values of the non-fixed nodes are written.
   */
  void readData(std::map<std::string, SArray> &data_table,
		bool observed) const;
};

#endif /* SYMTAB_H_ */
