#ifndef SARRAY_H_
#define SARRAY_H_

#include <sarray/Range.h>

/**
 * @short missing value
 */
extern const double JAGS_NA;

/**
 * SArrays represent multi-dimensional arrays of double precision floats
 * @short multi-dimensional array
*/
class SArray {
	
  Range _range;
  Index _dim1;
  Index _dim2;
  double *_value;
  bool  _fixed; 
  unsigned long _length;


public:
  /**
   * Constructor for SArrays. 
   * 
   * Upon construction, the elements of the value array are all
   * equal to JAGS_NA and the SArray is uninitialized.
   * @param dim Dimension of SArray to be constructed
   */
  SArray(Index const &dim);

  SArray(SArray const &orig);
  virtual ~SArray();

  SArray &operator=(SArray const &rhs);
  /** 
   * Set the SArray to be fixed (true) or not (false).
   *
   * Fixed SArrays cannot change their value. 
   * 
   * If fix == true and the SArray is uninitialized, it does not 
   * become fixed until after the next call to setValue.
   */
  void setFixed(bool fix);
	
  /**
   * Set value of SArray. 
   *
   * It is an error (logic_error) to set the value of
   * a fixed SArray.

   * @param value Array of values to be assigned
   * @param length Length of value argument. It is an error 
   *               (range_error) if this does not match the length 
   *               of the SArray.
   * @exception logic_error range_error
   */
  void setValue(double const *value, unsigned long length);

  /**
   * The value of the SArray in vector form. 
   *
   * Values are given in column order, like the S language (i.e. with
   * the left hand index moving fastest). 
   * @return A pointer to the beginning of the value array
   */
  inline double const *value() const;

  /** 
   * The dimension of the array
   * @param drop Following the S language convention, the drop
   *             argument determines whether dimensions of size 1
   *             should be dropped.
   * @return An index representing the dimension
   */
  Index const &dim(bool drop) const;

  /**
   * The length of the value vector: the product of the dimensions.
   */
  long length() const;
	
  /**
   * Indicates whether the SArray is fixed, and so cannot change
   * its value. This information may be useful when conducting
   * repeated calculations on SArrays.
   */
  bool isFixed() const;

  /**
   * Get the range of indices. The lower limit is always 1
   * for each element and the upper limit is equal to the dimension
   * of the SArray.
   */
  Range const &range() const;
};

inline double const *SArray::value() const
{
  return _value;
}

#endif /* SARRAY_H_ */
